==> critical-css-1.9.0-beta.0.tgz: package/src/extract-from-url.js <==
import {chromium} from 'playwright-chromium'
import CleanCSS from 'clean-css'
import {blockedResourceTypes, skippedResources} from './config.js'

let browser

const css = new CleanCSS({level: 2})

// Setup a browser instance or return the one already created
const getBrowser = async () => {
  if (browser) return browser

  browser = chromium.launch({
    args: [
      // Required for Docker version of Puppeteer
      '--no-sandbox',
      '--disable-gpu',
      '--disable-web-security',
      '--disable-setuid-sandbox',
      // This will write shared memory files into /tmp instead of /dev/shm,
      // because Docker’s default for /dev/shm is 64MB
      '--disable-dev-shm-usage'
    ]
  })

  return browser
}

export async function extractCSSFromUrl({
  customHeaders,
  height,
  url,
  userAgent,
  width
}) {
  let page
  try {
    // Get a browser instance
    browser = await getBrowser()
    const context = await browser.newContext({userAgent})
    // Create a new page and navigate to it
    page = await context.newPage()
    // Set viewport and user agent depending on the device
    await page.setViewportSize({width, height})
    page.setDefaultNavigationTimeout(15000)

    const hasCustomHeaders =
      typeof customHeaders === 'object' && Object.keys(customHeaders).length
    hasCustomHeaders && (await page.setExtraHTTPHeaders(customHeaders))

    await page.route('**/*', route => {
      const request = route.request()
      return blockedResourceTypes.includes(request.resourceType()) ||
        skippedResources.includes(request.url())
        ? route.abort().catch(() => {})
        : route.continue().catch(() => {})
    })

    await page.coverage.startCSSCoverage()

    const response = await page
      .goto(url, {waitUntil: 'load'})
      .catch(error => ({error}))

    const closeAll = async error => {
      await page.close()
      await browser.close()
      browser = null
      if (error) throw new Error(`[ko] ${error}`)
    }

    if (!response) await closeAll('Response is not present')

    if (!response.ok() && response.status() !== 304)
      await closeAll(`Response status code ${response.status()} for url ${url}`)

    console.log('[ok] Got response')

    const coverage = await page.coverage.stopCSSCoverage()
    console.log('[ok] Got coverage')

    let coveredCSS = ''
    for (const entry of coverage) {
      for (const range of entry.ranges) {
        coveredCSS += entry.text.slice(range.start, range.end)
      }
    }

    // Close the browser to close the connection and free up resources
    await closeAll()

    // return minified css
    const {styles, stats} = css.minify(coveredCSS)
    console.log(
      `[css] Minified from ${stats.originalSize} to ${stats.minifiedSize} bytes`
    )
    return styles
  } catch (e) {
    console.log(e)
    browser && browser.close && (await browser.close())
    browser = null
  }
}

==> critical-css-1.15.1-beta.0.tgz: package/src/extract-from-url.js <==
import {chromium} from 'playwright-chromium'
import css from '@parcel/css'
import {blockedResourceTypes, skippedResources} from './config.js'

let browser

// Setup a browser instance or return the one already created
const getBrowser = async () => {
  if (browser) return browser

  browser = chromium.launch({
    args: [
      // Required for Docker version of Puppeteer
      '--no-sandbox',
      '--disable-gpu',
      '--disable-web-security',
      '--disable-setuid-sandbox',
      // This will write shared memory files into /tmp instead of /dev/shm,
      // because Docker’s default for /dev/shm is 64MB
      '--disable-dev-shm-usage'
    ]
  })

  return browser
}

export async function extractCSSFromUrl({
  customHeaders,
  height,
  url,
  userAgent,
  width
}) {
  console.time(`[css] Extract for ${url}`)
  let page
  try {
    // Get a browser instance
    browser = await getBrowser()
    const context = await browser.newContext({userAgent})
    // Create a new page and navigate to it
    page = await context.newPage()
    // Set viewport and user agent depending on the device
    await page.setViewportSize({width, height})
    page.setDefaultNavigationTimeout(15000)

    const hasCustomHeaders =
      typeof customHeaders === 'object' && Object.keys(customHeaders).length
    hasCustomHeaders && (await page.setExtraHTTPHeaders(customHeaders))

    await page.route('**/*', route => {
      const request = route.request()
      return blockedResourceTypes.includes(request.resourceType()) ||
        skippedResources.includes(request.url())
        ? route.abort().catch(() => {})
        : route.continue().catch(() => {})
    })

    await page.coverage.startCSSCoverage()

    const response = await page
      .goto(url, {waitUntil: 'networkidle'})
      .catch(error => ({error}))

    const closeAll = async error => {
      await page.close()
      await browser.close()
      browser = null
      if (error) throw new Error(`[ko] ${error}`)
    }

    if (!response) await closeAll('Response is not present')

    if (response.error) await closeAll(response.error)

    if (!response.ok() && response.status() !== 304)
      await closeAll(`Response status code ${response.status()} for url ${url}`)

    console.log('[ok] Got response')

    const coverage = await page.coverage.stopCSSCoverage()
    console.log('[ok] Got coverage')

    let coveredCSS = ''
    for (const entry of coverage) {
      for (const range of entry.ranges) {
        coveredCSS += entry.text.slice(range.start, range.end)
      }
    }

    // Close the browser to close the connection and free up resources
    await closeAll()

    console.timeEnd(`[css] Extract for ${url}`)
    console.time('[css] Minified')
    // return minified css
    const {code} = css.transform({
      minify: true,
      code: Buffer.from(coveredCSS),
      sourceMap: false,
      targets: 'ie11'
    })

    const from = Buffer.byteLength(coveredCSS, 'utf-8')
    const to = Buffer.byteLength(code, 'utf-8')

    console.log(`[css] Minified from ${from} to ${to} bytes`)
    console.timeEnd('[css] Minified')
    return code
  } catch (e) {
    console.log(e)
    browser && browser.close && (await browser.close())
    browser = null
  }
}
